##############################################################################
#
# Copyright (c) 2003-2005 struktur AG and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
$Id: utils.py 14733 2005-12-08 13:23:19Z dreamcatcher $
"""

import sys
import httplib
import urlparse

class Connection(httplib.HTTPConnection):

    def __init__(self, host, port=None, strict=None, scheme="http", timeout=5):
        self.scheme = scheme
        if scheme == "http":
            self.default_port = httplib.HTTP_PORT
        elif scheme == "https":
            self.default_port = httplib.HTTPS_PORT
        else:
            raise ValueError, "Invalid scheme '%s'" % (scheme,)
        httplib.HTTPConnection.__init__(self, host, port, strict)
        self.timeout = timeout

    def connect(self):
        # We want to connect in normal blocking mode, else Python on Windows
        # will timeout even for 'connection refused' style errors (fixed in
        # Python 2.4 on Windows)
        if self.scheme == "http":
            httplib.HTTPConnection.connect(self)
        elif self.scheme == "https":
            # Clone of httplib.HTTPSConnection.connect
            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            sock.connect((self.host, self.port))
            key_file = cert_file = None
            ssl = socket.ssl(sock, key_file, cert_file)
            self.sock = httplib.FakeSocket(sock, ssl)
        else:
            raise ValueError, "Invalid scheme '%s'" % (self.scheme,)
        # Once we have connected, set the timeout.
        self.sock.settimeout(self.timeout)

XERROR = (
    'x-squid-error',
    'x-eep-error',
    )

def pruneUrl(url, purge_type='PURGE'):
    (scheme, host, path, params, query, fragment) = urlparse.urlparse(url)
    __traceback_info__ = (url, purge_type, scheme, host,
                          path, params, query, fragment)
    conn = Connection(host, scheme=scheme, timeout=2)
    try:
        # NOTE: only HTTP/1.0 can make PURGE work with Squid.
        # Apparently the full url is not needed with 2.5.STABLE12
        conn._http_vsn = 10
        conn._http_vsn_str = 'HTTP/1.0'
        conn.putrequest(purge_type, path)
        conn.endheaders()

        resp = conn.getresponse()
        status = resp.status
        xcache = resp.getheader('x-cache', '')
        for header in XERROR:
            xerror = resp.getheader(header, '')
            if xerror:
                # Break on first found.
                break
        resp.close()
    except:
        err, msg, tb = sys.exc_info()
        try:
            from zExceptions.ExceptionFormatter import format_exception
        except ImportError:
            from traceback import format_exception
        xerror = '\n'.join(format_exception(err, msg, tb))
        # Avoid leaking a ref to traceback.
        del err, msg, tb
        status = 000
        xcache = ''
    conn.close()
    return (status, xcache, xerror)

if __name__ == '__main__':
    print pruneUrl(sys.argv[1])
